﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.IO;
using System.Windows.Forms.DataVisualization.Charting;
using NationalInstruments.DAQmx;

namespace ketpaneles_meroprogram
{
    public partial class Form1 : Form
    {
        bool Run = false;
        StreamWriter fileWriter;
        NationalInstruments.DAQmx.Task InTask;
        AnalogSingleChannelReader analogReader;
        NationalInstruments.DAQmx.Task OutTask;
        AnalogSingleChannelWriter analogWriter;
        double SampleRate = 1000;
        bool direction=false;       //Motor direction: false->OUT, true->IN
        //bool enabled=true;          //Motor driver enabled

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            chart1.ChartAreas[0].AxisX.Title = "Time (sec)";
            chart1.ChartAreas[0].AxisY.Title = "Amplitude (V)";

            chart1.ChartAreas[0].AxisX.LabelStyle.Format = "0.##";
            chart1.ChartAreas[0].AxisY.LabelStyle.Format = "0.##";
            chart1.ChartAreas[0].AxisY.IsStartedFromZero = false;

            chart1.Legends[0].Docking = Docking.Top;

            chart1.Series[0].ChartType = SeriesChartType.Line;
            chart1.Series[0].Name = "Input";

            chart2.ChartAreas[0].AxisX.Title = "Time (sec)";
            chart2.ChartAreas[0].AxisY.Title = "Amplitude (V)";

            chart2.ChartAreas[0].AxisX.LabelStyle.Format = "0.##";
            chart2.ChartAreas[0].AxisY.LabelStyle.Format = "0.##";
            chart2.ChartAreas[0].AxisY.IsStartedFromZero = false;

            chart2.Legends[0].Docking = Docking.Top;

            chart2.Series[0].ChartType = SeriesChartType.Line;
            chart2.Series[0].Name = "Output";

            textBoxNumOfSteps.Text = "100";
        }

        private void buttonStart_Click(object sender, EventArgs e)
        {
            if (Run)
            {
                timer.Enabled = false;
                InTask.Dispose();
                OutTask.Dispose();
                Run = false;
                buttonStart.Text = "Start";
                fileWriter.Close();
            }
            else
            {
                saveFileDialog.ShowDialog();
            }
        }

        private void saveFileDialog_FileOk(object sender, CancelEventArgs e)
        {
            fileWriter = new StreamWriter(saveFileDialog.FileName);
            
            // Init myDAQ
            InTask = new NationalInstruments.DAQmx.Task();
            InTask.AIChannels.CreateVoltageChannel("myDAQ1/ai0", "",
                AITerminalConfiguration.Differential, -10, 10, AIVoltageUnits.Volts);
            InTask.Timing.ConfigureSampleClock("", SampleRate,
                SampleClockActiveEdge.Rising, SampleQuantityMode.ContinuousSamples);
            analogReader = new AnalogSingleChannelReader(InTask.Stream);

            OutTask = new NationalInstruments.DAQmx.Task();
            OutTask.AOChannels.CreateVoltageChannel("myDAQ1/ao1", "",
                -10, 10, AOVoltageUnits.Volts);
            OutTask.Timing.ConfigureSampleClock("", SampleRate,
                SampleClockActiveEdge.Rising, SampleQuantityMode.ContinuousSamples);
            analogWriter = new AnalogSingleChannelWriter(OutTask.Stream);

            // Generate data for output
            int numPnts = 5000;
            double[] outData = new double[numPnts];
            for (int i = 0; i < numPnts; i++)
            {
                outData[i] = Math.Sin((double) i / numPnts * 2 * Math.PI);
                chart2.Series[0].Points.AddXY(i, outData[i]);
            }
            
            // Start Input and output
            InTask.Start();
            analogWriter.WriteMultiSample(true, outData);

            Run = true;
            buttonStart.Text = "Stop";
            timer.Interval = Convert.ToInt32(numericInterval.Value);
            timer.Enabled = true;
        }

        private void timer_Tick(object sender, EventArgs e)
        {
            int SamplesToRead = Convert.ToInt32(Convert.ToDouble(numericInterval.Value)/1000 * SampleRate);
            double[] inData = new double[SamplesToRead];
            inData = analogReader.ReadMultiSample(SamplesToRead);

            chart1.Series[0].Points.Clear();
            for (int i = 0; i < SamplesToRead; i++)
            {
                double x = i;
                double y = inData[i];
                chart1.Series[0].Points.AddXY(x, y);
                fileWriter.WriteLine(Convert.ToString(x) + "\t" + Convert.ToString(y));
            }
        }

        private void Chart1ScaleChanged(object sender, EventArgs e)
        {
            if (Chart1AutoScale.Checked)
            {
                chart1.ChartAreas[0].AxisX.Minimum = Double.NaN;
                chart1.ChartAreas[0].AxisX.Maximum = Double.NaN;
                chart1.ChartAreas[0].AxisY.Minimum = Double.NaN;
                chart1.ChartAreas[0].AxisY.Maximum = Double.NaN;
            }
            else
            {
                chart1.ChartAreas[0].AxisX.Minimum = Convert.ToDouble(numericChart1XMin.Value);
                chart1.ChartAreas[0].AxisX.Maximum = Convert.ToDouble(numericChart1XMax.Value);
                chart1.ChartAreas[0].AxisY.Minimum = Convert.ToDouble(numericChart1YMin.Value);
                chart1.ChartAreas[0].AxisY.Maximum = Convert.ToDouble(numericChart1YMax.Value);
            }
        }

        private void Chart2ScaleChanged(object sender, EventArgs e)
        {
            if (Chart2AutoScale.Checked)
            {
                chart2.ChartAreas[0].AxisX.Minimum = Double.NaN;
                chart2.ChartAreas[0].AxisX.Maximum = Double.NaN;
                chart2.ChartAreas[0].AxisY.Minimum = Double.NaN;
                chart2.ChartAreas[0].AxisY.Maximum = Double.NaN;
            }
            else
            {
                chart2.ChartAreas[0].AxisX.Minimum = Convert.ToDouble(numericChart2XMin.Value);
                chart2.ChartAreas[0].AxisX.Maximum = Convert.ToDouble(numericChart2XMax.Value);
                chart2.ChartAreas[0].AxisY.Minimum = Convert.ToDouble(numericChart2YMin.Value);
                chart2.ChartAreas[0].AxisY.Maximum = Convert.ToDouble(numericChart2YMax.Value);
            }
        }

        private void buttonGetData_Click(object sender, EventArgs e)
        {
            int numPnts = chart1.Series[0].Points.Count;
            for (int i = 0; i < numPnts; i++)
            {
                DataPoint P = chart1.Series[0].Points.ElementAt(i);
                double x = P.XValue;
                double y = P.YValues[0];
            }
        }

        private void buttonClear1_Click(object sender, EventArgs e)
        {
            chart1.Series[0].Points.Clear();
        }

        private void buttonClear2_Click(object sender, EventArgs e)
        {
            chart2.Series[0].Points.Clear();
        }

        private void chart2_Click(object sender, EventArgs e)
        {

        }

        private void chart1_Click(object sender, EventArgs e)
        {

        }

        private void buttonSteps_Click(object sender, EventArgs e)
        {
            MotorStepFunction(Convert.ToInt16(textBoxNumOfSteps.Text),direction);
        }

        void MotorStepFunction(int Number, bool Direction)
        {
            //For stepper motor step adjust
            NationalInstruments.DAQmx.Task OutDigitalTask_Step;
            OutDigitalTask_Step = new NationalInstruments.DAQmx.Task();
            DigitalSingleChannelWriter Digital_DAQwriter_Step;

            //Output on DIO2 - STEP
            OutDigitalTask_Step.DOChannels.CreateChannel("myDaq1/port0/line2", "", ChannelLineGrouping.OneChannelForAllLines);
            Digital_DAQwriter_Step = new DigitalSingleChannelWriter(OutDigitalTask_Step.Stream);

            //For stepper motor step direction
            NationalInstruments.DAQmx.Task OutDigitalTask_Direction;
            OutDigitalTask_Direction = new NationalInstruments.DAQmx.Task();
            DigitalSingleChannelWriter Digital_DAQwriter_Direction;

            //Output on DIO1 - DIRECTION
            OutDigitalTask_Direction.DOChannels.CreateChannel("myDaq1/port0/line1", "", ChannelLineGrouping.OneChannelForAllLines);
            Digital_DAQwriter_Direction = new DigitalSingleChannelWriter(OutDigitalTask_Direction.Stream);
            Digital_DAQwriter_Direction.WriteSingleSampleSingleLine(true, Direction);

            //For stepper motor step adjust
            NationalInstruments.DAQmx.Task OutDigitalTask_Enable;
            OutDigitalTask_Enable = new NationalInstruments.DAQmx.Task();
            DigitalSingleChannelWriter Digital_DAQwriter_Enable;

            OutDigitalTask_Enable.DOChannels.CreateChannel("myDaq1/port0/line0", "", ChannelLineGrouping.OneChannelForAllLines);
            Digital_DAQwriter_Enable = new DigitalSingleChannelWriter(OutDigitalTask_Enable.Stream);
            Digital_DAQwriter_Enable.WriteSingleSampleSingleLine(true, checkBoxMotorEnabled.Checked);

            //Routine for steps
            for (int i = 0; i < Number; i++)
            {
                Digital_DAQwriter_Step.WriteSingleSampleSingleLine(true, true);
                System.Threading.Thread.Sleep(1);
                Digital_DAQwriter_Step.WriteSingleSampleSingleLine(true, false);
                System.Threading.Thread.Sleep(1);
            }
            OutDigitalTask_Step.Dispose();
            OutDigitalTask_Direction.Dispose();
            OutDigitalTask_Enable.Dispose();
        }

        private void radioButtonIN_CheckedChanged(object sender, EventArgs e)
        {
            direction = true;
        }

        private void radioButtonOUT_CheckedChanged(object sender, EventArgs e)
        {
            direction = false;
        }

        private void checkBoxMotorEnabled_CheckedChanged(object sender, EventArgs e)
        {

        }

    }
}
